#include <dm9pci.h>

extern FILE *dfd;

int dm9pci_scan( void *dll_hdl, io_net_self_t *ion, Nic_t *nic )
{
	Nic_Dm9pci_Ext_t	*ext;
	ushort_t		cmd;
	uint_t			cnt;
	uint_t			pidx;
	struct Config_Info	*cfg;
	struct pci_dev_info	inf;
	uint_t			dev_id[] = {	PCI_DEVICE_ID_DM9102,
						PCI_DEVICE_ID_DM9009,
						0 };
#if DEBUG_DETECT
fprintf(dfd, "dm9pci_scan()\n\r");
#endif
	ext = (Nic_Dm9pci_Ext_t *)nic->ext;

	for( cnt = 0; dev_id[cnt]; cnt++ ) {
		for( pidx = 0; pidx < 10; pidx++ ) {	
			memset( &inf, 0, sizeof( inf ) );

			cfg = &nic->cfg;
			inf.VendorId = ( cfg->Device_ID.DevID & 0x00ff ) ?
				cfg->Device_ID.DevID : PCI_VENDOR_ID_DAVICOM;
			inf.DeviceId = ( cfg->Device_ID.DevID & 0xff00 ) ?
				( cfg->Device_ID.DevID >> 16 ) : dev_id[cnt];

			pidx = nic->cfg.Device_ID.SerialNum ? nic->cfg.Device_ID.SerialNum : pidx;
			if( ( ext->hdl = pci_attach_device( NULL, PCI_INIT_ALL, pidx, &inf ) ) == NULL ) {
				if( nic->cfg.Device_ID.SerialNum || cfg->Device_ID.DevID ) {
					return( 1 );
				}
				continue;
			}
			if (inf.Class != PCI_CLASS_NETWORK) {
				pci_detach_device (ext->hdl);
				ext->hdl = NULL;
				continue;
			}

			ext->chip_id = dev_id [cnt];

			/* for distinquishing between E3 and E5 */
			pci_read_config( ext->hdl, 0x50, 1, sizeof(ext->pci_pmr), &ext->pci_pmr);

			if ( pci_read_config( ext->hdl, offsetof (struct _pci_config_regs, Command), 1, sizeof( cmd ), &cmd ) == -1 ) {
				return( 1 );
			}

			cmd |= PCI_COMMAND_MASTER_ENABLE;
			if ( pci_write_config( ext->hdl, offsetof (struct _pci_config_regs, Command), 1, sizeof( cmd ), &cmd ) == -1) {
				return( 1 );
			}

			cfg->Device_ID.DevID	= inf.VendorId | ( inf.DeviceId << 16 );
			cfg->BusAccess.PCIAccess.BusNumber	= inf.BusNumber;
			cfg->BusAccess.PCIAccess.DevFuncNumber	= inf.DevFunc;
			cfg->NumIRQs		= 1;
			cfg->IRQRegisters[0]	= cfg->IRQRegisters[0] ? cfg->IRQRegisters[0] : inf.Irq;
			cfg->NumIOPorts		= 1;
			cfg->IOPort_Base[0]	= PCI_IO_ADDR( inf.CpuBaseAddress[0] );
			cfg->IOPort_Length[0]	= inf.BaseAddressSize[0];
			ext->bmstr		= inf.CpuBmstrTranslation;

			if( !dm9pci_register_device( nic, ion, dll_hdl ) ) {
				dm9pci_advertise(ext->reg_hdl, nic);
				return( 0 );
			}
			pci_detach_device( ext->hdl );
			return( 1 );
		}
	}

	return( 1 );
}

int dm9pci_args( void *dll_hdl, io_net_self_t *ion, char *options )
{
	int			rval;
	Nic_t		*nic;
	Nic_Dm9pci_Ext_t	*ext;

#if DEBUG_DETECT
fprintf(dfd, "dm9pci_args()\n\r");
#endif
	if( ( nic = nic_create_dev( sizeof( Nic_Dm9pci_Ext_t ) ) ) == NULL ) {
		return( ENOMEM );
	}

	ext = (Nic_Dm9pci_Ext_t *)nic->ext;

	/* prime this so we know if mtu is given on the command line */
	nic->mtu = 0;

	if( rval = dm9pci_scan( dll_hdl, ion, nic ) ) {
		free( nic->ext );
		free( nic );
	}
	return( rval );
}

int dm9pci_detect( void *dll_hdl, io_net_self_t *ion, char *options )
{
	int		cnt;
	Nic_t	*nic;
	int		found = NIC_FALSE;

#if DEBUG_DETECT
fprintf(dfd, "dm9pci_detect()\n\r");
#endif
	if( pci_attach( 0 ) == -1 ) {
		fprintf( stderr, "dm9pci_detect:  Unable to locate pci server\n\r" );
		return( ENODEV );
	}

	if( options && *options != '-') {
		return( dm9pci_args( dll_hdl, ion, options ) );
	}

	for( cnt = 0 ; cnt < 20; cnt++ ) {
		if( nic = nic_create_dev( sizeof( Nic_Dm9pci_Ext_t ) ) ) {
			if( !dm9pci_scan( dll_hdl, ion, nic ) ) {
				found = NIC_TRUE;
				continue;
			}
			free( nic->ext );
			free( nic );
			break;
		}
	}
	return( found ? EOK : ENODEV );
}

/** PhEDIT attribute block
0:3448:default:-3:-3:0
**  PhEDIT attribute block ends (-0000096)**/
