#ifndef NIC_DM9PCI_H_INCLUDED
#define NIC_DM9PCI_H_INCLUDED

/*
 * dm9pci.h
 * Copyright (c) 2000 DAVICOM Semiconductor Inc. All Rights Reserved 
 *
 * This file contains the hardware definitions for addressing
 * DAVICOM DM9PCI Series Fast Ethernet adapter.
 *
 * Author: John H.T. Ho
 *
 * Modification History
 *
 * Date	      Modfier.	 Modification Descriptions
 * ---------- ---------- ------------------------------------------
 * 2000.6.12  htho	 new today
 */

#include <time.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>
#include <atomic.h>
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/siginfo.h>
#include <sys/syspage.h>
#include <sys/neutrino.h>
#include <hw/inout.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <sys/io-net.h>
#include <sys/dcmd_io-net.h>

#include <drvr/mdi.h>
#include <eth.h>
#include <hw/pci.h>
#include <hw/pci_devices.h>

#include <drvr/support.h>

#define PCI_DEVICE_ID_DM9102  0x9102
#define PCI_DEVICE_ID_DM9009  0x9009
#define PCI_VENDOR_ID_DAVICOM 0x1282

#define TIMER_ACTIVE	1

#define CONSOLE_DEBUG	0	/* open /dev/ser1 to print debug message */
#define DEBUG_DETECT	0	/* NIC detect flow while mount the device */
#define	DEBUG_INIT	0	/* init procedure */
#define	DEBUG_AUTO	0	/* auto detect media type */
#define	DEBUG_TX	0	/* Tx flow */
#define DEBUG_RX	0	/* Rx flow */
#define DEBUG_INT	0	/* interrupt handling */
#define DEBUG_TIMER	0	/* timer service routine */

// Bytes transferred to chip before transmission starts.
#define TX_FIFO_THRESH 128		// Rounded down to 4 byte units.
#define RX_FIFO_THRESH 1		// 0-3, 0==32, 64,96, or 3==128 bytes


/************************************************************************
 *
 *			DM9PCI Hardware Definitions
 *
 ************************************************************************/
#define RX_DESC_NUM		30
#define TX_DESC_NUM		15

#define BUFFER_SIZE			2048L	/* Enough to hold a 1536 Frame     */
#define MAX_FRAME_SIZE	1536		/* 1536                            */
#define MAX_MULTI			32			/* Hardware Related, Do not change */

#define ETHER_ADDR_LEN		6

#define CSR0_DEFAULT		0x00C00000
#define CSR6_DEFAULT		0x02040000
//#define CSR7_DEFAULT		0x00018041      /* CSR7_NISE|CSR7_AISE|CSR7_RXCIE|CSR7_TXCIE */
#define CSR7_DEFAULT		0x00018041      /* CSR7_NISE|CSR7_AISE|CSR7_RXCIE|CSR7_TXCIE */
#define CSR15_DEFAULT		0x06            /* TxJabber RxWatchdog */

#define TDES0_ERR_MASK		0x4302          /* TXJT, LC, EC, FUE */
#define MAX_PACKET_SIZE		1514
#define DMFE_MAX_MULTICAST	14
#define RX_MAX_TRAFFIC		0x14000
#define MAX_CHECK_PACKET	0x8000

#define MEDIA_TYPE_10HDX	0
#define MEDIA_TYPE_100HDX	1
#define MEDIA_TYPE_10FDX	4
#define MEDIA_TYPE_100FDX	5
#define MEDIA_TYPE_AUTO		8

#define MEDIA_TYPE_1HOMEPNA	0x10

#define HPNA_NOT_PRESENT	0
#define	HPNA_HOMERUN		1
#define HPNA_LONGRUN		2

#define LINK_PASS		0
#define LINK_FAIL		1

/*
 * DM9PCI Control Register Offset Definitions
 */
#define CSR0		0x00
#define CSR1		0x08
#define CSR2		0x10
#define CSR3		0x18
#define CSR4		0x20
#define CSR5		0x28
#define CSR6		0x30
#define CSR7		0x38
#define CSR8		0x40
#define CSR9		0x48
#define CSR10		0x50
#define CSR11		0x58
#define CSR12		0x60
#define CSR13		0x68
#define CSR14		0x70
#define CSR15		0x78

/*
 * System Control Register (CSR0) Bit Definitions
 */
#define CSR0_SR		0x00000001	/* Software Reset */

/*
 * Network Status Report Register (CSR5) Bit Definitions
 */
#define CSR5_TXCI	0x00000001	/* Transmit Complete Interrupt */
#define CSR5_TXPS	0x00000002	/* Transmit Process Stopped */
#define CSR5_TXDU	0x00000004	/* Transmit Buffer Unavailable */
#define CSR5_TXJT	0x00000008	/* Transmit Jabber Timer Expired */
#define CSR5_TXFU	0x00000020	/* Transmit FIFO Underrun */
#define CSR5_RXCI	0x00000040	/* Receive Complete Interrupt */
#define CSR5_RXDU	0x00000080	/* Receive Buffer Unavailable */
#define CSR5_RXPS	0x00000100	/* Receive Process Stopped */
#define CSR5_RXWT	0x00000200	/* Receive Watchdog Timer Expired */
#define CSR5_TXER	0x00000400	/* Transmit Early Interrupt */
#define CSR5_GPT	0x00000800	/* General Purpose Timer Expired */
#define CSR5_SBE	0x00002000	/* System Bus Error */
#define CSR5_AIS	0x00008000	/* Abnormal Interrupt Summary */
#define CSR5_NIS	0x00010000	/* Normal Interrupt Summary */

#define CSR5_M_PXPS	0x000E0000	/* Receive Process State mask */

 
/*
 * Network Operation Mode Register (CSR6) Bit Definitions
 */
#define CSR6_HPFM	0x00000001	/* Hash/Perfect Filter Mode */
#define CSR6_RXSC	0x00000002	/* Receive start/stop command */
#define CSR6_HOFM	0x00000004	/* Hash-only filter mode */
#define CSR6_PBF	0x00000008	/* Pass bad frame */
#define	CSR6_IAFM	0x00000010	/* Inverse address filtering mode */
#define CSR6_PM		0x00000040	/* Promiscous mode */
#define CSR6_PAM	0x00000080	/* Pass all multicast */
#define CSR6_FDM	0x00000200	/* Full duplex mode */
#define CSR6_FCM	0x00001000	/* Force collision mode */
#define CSR6_TXSC	0x00002000	/* Transmit start/stop command */
#define CSR6_PS		0x00040000	/* Port select; 0=internal ,1=external */
#define CSR6_STI	0x00100000	/* Start transmit immediately */
#define CSR6_SFT	0x00200000	/* Store and forward transmit */
#define CSR6_TXTM	0x00400000	/* Transmit threshold mode */
#define CSR6_RXA	0x40000000	/* Receive all */

#define CSR6_M_LBM		0x00000C00	/* Loopback mode mask*/
#define CSR6_LBM_NORMAL		0x00000000	/* normal */
#define	CSR6_LBM_INTERNAL	0x00000400	/* internal loopback */
#define CSR6_LBM_PHY_DIGIT	0x00000800	/* PHY digit loopback */
#define CSR6_LBM_PHY_ANALOG	0x00000C00	/* PHY analog loopback */

#define CSR6_M_TSB		0x0000C000	/* Threshold bits mask */
#define CSR6_TSB_128		0x00000000
#define CSR6_TSB_256		0x00004000
#define CSR6_TSB_512		0x00008000

/*
 * Interrupt Mask Register (CSR7) Bit Definitions
 */
#define CSR7_NISE	0x000010000	/* Normal Interrupt Summary Enable */
#define CSR7_AISE	0x000008000	/* Abnormal Interupt Summary Enable */
#define CSR7_SBEE	0x000002000	/* System Bus Error Enable */
#define CSR7_GPTE	0x000000800	/* General-Purpose Timer Expired Enable */
#define CSR7_ETIE	0x000000400	/* Transmit Early Interrupt Enable */
#define CSR7_RXWTE	0x000000200	/* Receive Watchdog Timer Expired Enable */
#define CSR7_RXPSE	0x000000100	/* Receive Process Stopped Enable */
#define CSR7_RXDUE	0x000000080	/* Receive Buffer Unavailable Enable */
#define CSR7_RXCIE	0x000000040	/* Receive Complete Interrupt Enable */
#define CSR7_RXFUE	0x000000020	/* Receive FIFO Underrun Enable */
#define CSR7_TXFUE	0x000000008	/* Transmit FIFO Underrun Enable */
#define CSR7_TXJTE	0x000000004	/* Transmit Jabber Timer Expired Enable */
#define CSR7_TXPSE	0x000000002	/* Transmit Process Stopped Enable */
#define CSR7_TXCIE	0x000000001	/* Transmit Complete Interrupt Enable */

/* CR9 definition: SROM/MII */
#define CSR9_SROM_READ	0x4800
#define CSR9_SRCS	0x1
#define CSR9_SRCLK	0x2
#define CSR9_CRDOUT	0x8
#define SROM_DATA_0	0x0
#define SROM_DATA_1	0x4
#define PHY_DATA_1	0x20000
#define PHY_DATA_0	0x00000
#define MDCLKH		0x10000

/* CR12 Bit Definitions */
#define CSR12_GEPC	0x00000100	/* GEPD bits control */
#define CSR12_GEPD	0x00000080	/* General PHY reset control */

/* Rx Descriptor Bit Definitions */
#define RDES0_OWN	0x80000000	/* OWN bit 0=HOST 1=MAC */
#define RDES0_M_LENGTH	0x7FFF0000	/* Length of frame in bytes including CRC */
#define RDES0_ES	0x00008000	/* Error Summary (DUE|RF|EFL|LCS|CRC|FOE) */
#define RDES0_DUE	0x00004000	/* Descriptor Unavailable Error */
#define RDES0_M_LBOM	0x00003000	/* Loopback operation mode */
#define RDES0_RF	0x00000800	/* Runt Frame */
#define RDES0_MF	0x00000400	/* Multicat frame */
#define RDES0_BD	0x00000200	/* Begin Descriptor */
#define RDES0_ED	0x00000100	/* End Descriptor */
#define RDES0_EFL	0x00000080	/* Excessive Frame Length (>1518 bytes) */
#define RDES0_LCS	0x00000040	/* Late Collision Seen */
#define RDES0_FT	0x00000020	/* Frame Type 1=DIX 0=802.3 */
#define RDES0_RXWT	0x00000010	/* Receive Watchdog Expired (frame >2K) */
#define RDES0_PLE	0x00000008	/* Physical Layer Error */
#define RDES0_AE	0x00000004	/* Alignment Error */
#define RDES0_CE	0x00000002	/* CRC Error */
#define RDES0_FOE	0x00000001	/* FIFO Overflow Error */

#define RDES1_EOR	0x02000000	/* End Of Ring */
#define RDES1_CE	0x01000000	/* Chain Enable */
#define RDES1_M_BL2	0x003FF800	/* Size in bytes of buffer 2 (MOD 4) */
#define RDES1_M_BL1	0x000007FF	/* Size in bytes of buffer 1 (MOD 4) */

/* Rx Descriptor Bit Definitions */
#define TDES0_OWN	0x80000000	/* OWN Bit 1=MAC 0=Host */
#define TDES0_ES	0x00008000	/* Error Summary (TXJT|LOC|NC|LC|EC|FUE) */
#define TDES0_TXJT	0x00004000	/* Transmit Jabber Expired */
#define TDES0_LOC	0x00000800	/* Loss of Carrier */
#define TDES0_NC	0x00000400	/* No Carrier */
#define TDES0_LC	0x00000200	/* Late Collision */
#define TDES0_EC	0x00000100	/* Excessive Collisions */
#define TDES0_M_CC	0x00000078	/* Collision Count */
#define TDES0_FUE	0x00000002	/* FIFO Underflow Error */
#define TDES0_DF	0x00000001	/* Deferred */

#define TDES1_CI	0x80000000	/* Completion Interrupt */
#define TDES1_ED	0x40000000	/* End Descriptor */
#define TDES1_BD	0x20000000	/* Begin Descriptor */
#define TDES1_FMB1	0x10000000	/* Filtering Mode Bit 1 */
#define TDES1_SETF	0x08000000	/* Setup Frame */
#define TDES1_CAD	0x04000000	/* CRC Append Disable */
#define TDES1_EOR	0x02000000	/* End of Ring */
#define TDES1_CE	0x01000000	/* Chain Enable */
#define TDES1_PD	0x00800000	/* Padding Disable */
#define TDES1_FMB0	0x00400000	/* Filtering Mode Bit 0 */
#define TDES1_M_BL2	0x003FF800	/* Size in bytes of Buffer 2 */
#define TDES1_M_BL1	0x000007FF	/* Size in bytes of Buffer 1 */


/* PHY Register Definitions */
#define PHY_BMCR	0x00		/* Basic Mode Control Register */
#define PHY_BMSR	0x01		/* Basic Mode Status Register */
#define PHY_PHYIDR1	0x02		/* PHY Identifier Register #1 */
#define PHY_PHYIDR2	0x03		/* PHY Identifier Register #2 */
#define PHY_ANAR	0x04		/* Auto-Negotiation Advertisement Register */
#define PHY_ANLPAR	0x05		/* Auto-Negotiation Link Partner Ability Reg. */
#define PHY_ANER	0x06		/* Auto-Negotiation Expansion Register */
#define PHY_DCR		0x10		/* DAVICOM Special Config Register */
#define PHY_DSCR 	0x11		/* DAVICOM Special Config/Status Reg. */
#define PHY_10BTSCR	0x12		/* 10BASE-T Configuration/Status Reg. */

#define BMCR_RST	0x8000		/* Reset */
#define BMCR_LB		0x4000		/* loopback */
#define BMCR_SL		0x2000		/* Speed Select: 1=100M, 0=10M */
#define BMCR_ANE	0x1000		/* Auto-Negotiation Enable */
#define BMCR_IS		0x0400		/* Isolate */
#define BMCR_RAN	0x0200		/* Restart Auto-Negotiation */
#define BMCR_DM		0x0100		/* Duplex Mode: 1=Full, 0=Half */
#define BMCR_CT		0x0080		/* Collision Test */

#define BMSR_T4		0x8000		/* 100BASE-T4 Capable */
#define BMSR_TX_FDX	0x4000		/* 100BASE-TX Full Duplex Capable */
#define BMSR_TX_HDX	0x2000		/* 100BASE-TX Half Duplex Capable */
#define BMSR_10_FDX	0x1000		/* 10BASE-T Full Duplex Capable */
#define BMSR_10_HDX	0x8000		/* 10BASE-T Full Duplex Capable */
#define BMSR_PS		0x0040		/* MII Frame Preamble Supression */
#define BMSR_ANC	0x0020		/* Auto-Negotiation Complete */
#define BMSR_RF		0x0010		/* Remote Fault */
#define BMSR_ANA	0x0008		/* Auto-Negotiation Ability */
#define BMSR_LS		0x0004		/* Link Status: 1=link established */
#define BMSR_JD		0x0002		/* Jabber Detect */
#define BMSR_EC		0x0001		/* Extended Capability */

#define ANAR_RF		0x2000		/* Remote Fault */
#define ANAR_FCS	0x0400		/* Flow Control Support */
#define ANAR_T4 	0x0200		/* 100BASE-T4 Support */
#define ANAR_TX_FDX	0x0100		/* 100BASE-TX Full Duplex support */
#define ANAR_TX_HDX	0x0080		/* 100BASE-TX Half Duplex support */
#define ANAR_10_FDX	0x0040		/* 10BASE-T Full Duplex support */
#define ANAR_10_HDX	0x0020		/* 10BASE-T Half Duplex support */

#define ANAR_M_PS	0x001F		/* Protocol Selection mask */
#define ANAR_CSMACD	0x0001		/* device supports IEEE 802.3 CSMA/CD */

#define ANAR_M_CAP	0x03E0		/* advertisement capability mask ANAR[5..9] */
#define ANAR_CAP_ALL	(ANAR_TX_FDX|ANAR_TX_HDX|ANAR_10_FDX|ANAR_10_HDX)

#define ANLPAR_RF	0x2000		/* Remote Fault */
#define ANLPAR_T4 	0x0200		/* 100BASE-T4 Support */
#define ANLPAR_TX_FDX	0x0100		/* 100BASE-TX Full Duplex support */
#define ANLPAR_TX_HDX	0x0080		/* 100BASE-TX Half Duplex support */
#define ANLPAR_10_FDX	0x0040		/* 10BASE-T Full Duplex support */
#define ANLPAR_10_HDX	0x0020		/* 10BASE-T Half Duplex support */

#define ANLPAR_M_CAP	0x03E0		/* partner capability mask ANLPAR[5..9] */

#define ANLPAR_M_PS	0x001F		/* Protocol Selection mask */
#define ANLPAR_CSMACD	0x0001		/* device supports IEEE 802.3 CSMA/CD */

#define ANER_LP_AN_ABLE	0x0001		/* Link Partner Auto-Negotiation Able */

#define DSCR_M_OM	0xF000		/* Operation Mode Mask */
#define DSCR_100FDX	0x8000		/* 100M Full Duplex Mode */
#define DSCR_100HDX	0x4000		/* 100M Half Duplex Mode */
#define DSCR_10FDX	0x2000		/* 10M Full Duplex Mode */
#define DSCR_10HDX	0x1000		/* 10M Full Duplex Mode */

/* Delay Macros Definitions */
#define DELAY_5US	nsec_delay(5000)	/* delay 5 usec */
#define DELAY_1US	nsec_delay(1000)	/* delay 1 usec */

/* Structure/enum declaration ------------------------------- */
typedef struct 
{
	uint32_t	tdes0, tdes1, tdes2, tdes3;
	uint32_t	tx_npkt_ptr;
	uint32_t	tx_buf_ptr;
	uint32_t	next_tx_desc;
	uint32_t	reserved;
 } dm9pci_tx_desc;

typedef struct
{
	uint32_t	rdes0, rdes1, rdes2, rdes3;
	uint32_t	rx_npkt_ptr;
	uint32_t	rx_buf_ptr;
	uint32_t	next_rx_desc;
	uint32_t	reserved;
}dm9pci_rx_desc;

#define COMMAND			0x00
	#define TXUGO		0x80
	#define STOP_RDMA	0x40
	#define STOP_TDMA	0x20
	#define NEXTFRAME	0x10
	#define RXQUEUED	0x08
	#define TXQUEUED	0x04
	#define START_RX	0x02
	#define STOP_RX		0x01

#define INTR_STAT		0x04
#define INTR_MASK		0x08
#define CONTROL			0x0c
#define	NVCTL			0x10
#define EEPROM_CTL		0x14
#define TEST1			0x1c
#define CRC_ERR_CNT		0x20
#define ALIGN_CNT		0x24
#define OVER_CNT		0x28
#define MII_CONTROL		0x30
#define MII_DATA		0x34
#define	MII_CONFIG		0x38
#define LAN0			0x40
#define MC0			    0x50
#define MC1			    0x54
#define MC2			    0x58
#define MC3			    0x5C
#define RX_CONTROL		0x60
#define TX_CONTROL		0x70
#define RX_BASE			0x84
#define TX_BASE			0xc4
#define TX_THRESH		0xdc

/*
 * I/O Macro Definitions
 */
#define OUT8(ext, offset, value)	out8((ext->iobase+(offset)), (value))
#define OUT6(ext, offset, value)	outle16((ext->iobase+(offset)), (value))
#define OUT32(ext, offset, value)	outle32((ext->iobase+(offset)), (value))
#define IN8(ext, offset)		(in8(ext->iobase+(offset)))
#define IN16(ext, offset)		(inle16(ext->iobase+(offset)))
#define IN32(ext, offset)		(inle32(ext->iobase+(offset)))

#define RNDUP(x, y) (((x) + ((y) - 1)) & ~((y) - 1))

// Interrupt register bits


#define TX_IDLE			0x40000
#define RX_IDLE			0x20000
#define CNT_FULL		0x200
#define TX_UNDERRUN		0x100
#define TX_EMPTY		0x80
#define TX_DONE			0x20
#define RX_ERROR		0x10
#define RX_OVERFLOW		0x8
#define RX_FULL			0x4
#define RX_HEADER		0x2
#define RX_DONE			0x1			

#define DM9PCI_MULTICAST_ENTRIES		8
#define DM9PCI_ISR_VALID	( CNT_FULL | TX_UNDERRUN | TX_DONE | RX_ERROR | RX_OVERFLOW | RX_DONE )

#define DM9PCI_RX_FRAGMENTS		2
//#define DM9PCI_TX_FRAGMENTS		16
#define DM9PCI_TX_FRAGMENTS		1

#define TFD_STAT_OWNER			0x8000
#define TFD_CTL_LASTDESCR		0x10
#define TFD_CTL_NOCRC			0x08
#define TFD_CTL_IAF				0x04
#define TFD_CTL_LFFORM			0x02
#define TFD_CTL_FRAGLIST		0x01

#define RFD_STAT_OWNER			0x8000
#define TFD_CTL_HEADER			0x04
#define RFD_CTL_LFFORM			0x02
#define RFD_CTL_FRAGLIST		0x01

typedef struct fragment {
	ulong_t		b_addr;
	ulong_t		length;
} Frag_t;

// Transmit frame descriptor
typedef struct TFD {
	ushort_t	status;
	ushort_t	framelen;
	ulong_t		b_addr;
	ushort_t	b_bufsize;
	ushort_t	control;
	ulong_t		link;
	ulong_t		nfrag;
	Frag_t		frag[DM9PCI_TX_FRAGMENTS];
} TFD_t;

// Receive frame descriptor
typedef struct RFD {
	ushort_t	status;
	ushort_t	framelen;
	ulong_t		b_addr;
	ushort_t	b_bufsize;
	ushort_t	control;
    ulong_t		link;
#ifdef DM9PCI_RFD_FRAGMENTS
	ulong_t		nfrag;
	Frag_t		frag[DM9PCI_RX_FRAGMENTS];
#endif
} RFD_t;

/*
 * 48-bit ethernet address, passed back from ETHER_GETADDR
 */
typedef struct {
	unsigned char ebyte[6];
} ether_address_t;

typedef struct _nic_dm9pci_ext {
	Nic_t		*nic;
	int		eflags;
	int		chid;		// channel id
	int		coid;		// connection id
	int		tid;		// thread id
	int		iid;		// interrupt id
	io_net_self_t	*ion;
	void		*dll_hdl;
	int		reg_hdl;
	uint16_t	cell;
	uint16_t	zero;
	unsigned	max_pkts;
	pthread_mutex_t	mutex;
	mdi_t		*mdi;
	void		*hdl;		// pci device handle
	int		istat;
	int		negotiate;
	int		verbose;
	int		iobase;
	int		bmstr;
	unsigned	num_rx_free;
	npkt_t		*nhead;
	npkt_t		*ntail;
	npkt_t		*npkt;
	npkt_t		**rx_pktq;
	npkt_t		**tx_pktq;
	int		rx_pidx;	// last rx buffer produced
	int		rx_cidx;	// last rx buffer consumed
	volatile RFD_t	*rfd;
	volatile TFD_t	*tfd;
	int		tx_pidx;	// last command produced
	int		tx_cidx;	// last command comsumed
	npkt_t		*rx_free;
	int		chip_id;

	/*
	 * Rings
	 */
//	dm9pci_tx_desc	tx_desc[TX_DESC_NUM];	/* TX Frame Descriptor                  */
//	dm9pci_rx_desc	rx_desc[RX_DESC_NUM];	/* RX Frame Descriptor                  */
	unsigned char *alloc_tx_desc;		/* TX Descriptor pool pointer */
	unsigned char *alloc_rx_desc;		/* RX Descriptor pool pointer */
	dm9pci_tx_desc *tx_desc;			/* pointer to Tx Descriptor list */
	dm9pci_rx_desc *rx_desc;			/* pointer to Rx Descriptor list */
	unsigned char	*alloc_tx_buf;		/* TX Buffer                            */
	unsigned char	*alloc_rx_buf;		/* RX Buffer                            */
	unsigned char	*tx_buf;		/* TX Buffer                            */
	unsigned char	*rx_buf;		/* RX Buffer                            */

	ether_address_t	macaddr;	/* Ethernet Address		  */

	int	cur_tx;
	int	cur_rx;
	int	last_tx;

	uint32_t	cr0_data;
	uint32_t	cr5_data;
	uint32_t	cr6_data;
	uint32_t	cr7_data;
	uint32_t	cr15_data;

	uint16_t	anar;					/* Auto Negotiation Ability Register */

	uint8_t	cfg_mode;			/* user specify media mode */
	uint8_t	op_mode;			/* real work media mode */
	uint8_t	phy_addr;
	uint8_t	HPNA_type;
	uint8_t	link_status;

	uint8_t	open;					/* indicate if NIC is opened or not */

	uint32_t	pci_pmr;				/* for distiquishing between E3 and E5 */
	
	unsigned char srom[128];
    NicTxRxCount_t	txrx_count;
} Nic_Dm9pci_Ext_t;

void *dm9pci_driver_thread( void *data );
npkt_t *dm9pci_alloc_npkt( Nic_t *nic, size_t size );

int dm9pci_transmit_complete( Nic_t *nic );

int dm9pci_init( void *, dispatch_t *dpp, io_net_self_t *, char *options);
int dm9pci_send_packets( npkt_t *npkt, void * );
int dm9pci_receive( Nic_t *nic );
int dm9pci_receive_complete( npkt_t *npkt, void *, void * );
int dm9pci_devctl( void *hdl, int dcmd, void *data, size_t size, int *ret );
int dm9pci_shutdown1(int, void *);
int dm9pci_shutdown2(int, void *);
int dm9pci_advertise(int reg_hdl, void *func_hdl);
int dm9pci_flush( int, void *hdl );

int dm9pci_detect( void *, io_net_self_t *, char *options );
int dm9pci_register_device( Nic_t *nic, io_net_self_t *, void *);

extern paddr_t mphys( void * );

void nsec_delay( int nsec );

#define ion_rx_packets          ext->ion->tx_up
#define ion_register            ext->ion->reg
#define ion_deregister          ext->ion->dereg
#define ion_alloc               ext->ion->alloc
#define ion_free                ext->ion->free
#define ion_alloc_npkt          ext->ion->alloc_up_npkt
#define ion_mphys               ext->ion->mphys
#define ion_tx_complete         ext->ion->tx_done
#define ion_add_done            ext->ion->reg_tx_done

#define TAILQ_EMPTY(head) ((head)->tqh_first == NULL)
#define TAILQ_FIRST(head) ((head)->tqh_first)
#define TAILQ_LAST(head) ((head)->tqh_last)
#define TAILQ_NEXT(elm, field) ((elm)->field.tqe_next)
#define TAILQ_PREV(elm, field) ((elm)->field.tqe_prev)

#endif

/** PhEDIT attribute block
0:19807:default:-3:-3:0
**  PhEDIT attribute block ends (-0000097)**/
