#include <dm9pci.h>

extern FILE *dfd;
extern void dump_tx_desc(Nic_Dm9pci_Ext_t *ext);

void dump_npkt(npkt_t *npkt);
int copy_npkt_data(void *addr, npkt_t *npkt);

npkt_t *dm9pci_defrag( Nic_t *nic, npkt_t *npkt )
{
	npkt_t                          *dpkt;
	char                            *dst;
	net_iov_t                       *iov;
	net_buf_t                       *dov, *buf;
	int                             totalLen;
	int                             i, len;

	Nic_Dm9pci_Ext_t	*ext;
#if DEBUG_TX
fprintf(dfd, "dm9pci_defrag()\n\r");
#endif
	ext		= (Nic_Dm9pci_Ext_t *)nic->ext;
	dpkt = NULL;

	/* Do some checks */
	for( totalLen = 0, buf = TAILQ_FIRST(&npkt->buffers); buf != NULL;
		buf = TAILQ_NEXT( buf, ptrs ) ) {
		for( i = 0, iov = buf->net_iov; i < buf->niov; i++, iov++ ) {
			totalLen += iov->iov_len;
		}
	}

	if( totalLen <= 1520 ) 
	{
		if((dpkt = dm9pci_alloc_npkt(nic, npkt->framelen)) != NULL) 
		{ 
			dpkt->framelen  = npkt->framelen;
			dpkt->flags = _NPKT_NO_RES;
			dpkt->next = NULL;

			dov = TAILQ_FIRST(&dpkt->buffers);
			/* Realign to dword */
//			dov->net_iov->iov_base = (char *)(dov->net_iov->iov_base) - 2;
//			dov->net_iov->iov_phys -= 2;
			dst = dov->net_iov->iov_base;
			for( len = 0, buf = TAILQ_FIRST( &npkt->buffers ); buf != NULL;
				 buf = TAILQ_NEXT( buf, ptrs ) ) 
			{
				for( i = 0, iov = buf->net_iov; i < buf->niov; i++, iov++ ) 
				{
					memcpy( dst, iov->iov_base, iov->iov_len );
					dst += iov->iov_len;
					len += iov->iov_len;
				}
			}

			dov->net_iov->iov_len = len;
		}
	}

	if( npkt->flags & _NPKT_NO_RES )
	{
		ion_free( npkt->org_data );
		ion_free( npkt );
	}
	else
	{
		pthread_mutex_unlock( &ext->mutex );
		ion_tx_complete(ext->reg_hdl, npkt);
		pthread_mutex_lock( &ext->mutex );
	}

	return (dpkt);
}

int dm9pci_send( Nic_t *nic )
{
	dm9pci_tx_desc *tfd;
	int		pidx;
	int		wakeup;
	int		framelen;
	npkt_t		*npkt;
	void		*addr;
	Nic_Dm9pci_Ext_t	*ext;

#if DEBUG_TX
fprintf(dfd, "dm9pci_send()\n\r");
#endif
	ext = (Nic_Dm9pci_Ext_t *)nic->ext;
	wakeup = 0;

	/* load up the card */
	while( npkt = ext->nhead ) {
		pidx = (ext->tx_pidx + 1) % TX_DESC_NUM;

		/* check if there are free Tx descriptors */
		if ( ext->tx_pktq[pidx] )
			break;		/* no space */

		ext->nhead = ext->nhead->next;
		npkt->next = NULL;
		if (npkt == ext->ntail)
			ext->ntail = NULL;
		
		ext->tx_pidx = pidx;
		ext->tx_pktq[pidx] = npkt;

		addr = &ext->tx_buf[pidx * BUFFER_SIZE];
		framelen = copy_npkt_data(addr, npkt);
#if DEBUG_TX
fprintf(dfd, "transmit data length = %d\n\r", framelen);
#endif
        ext->txrx_count.tx_bytes += framelen;

		tfd = (dm9pci_tx_desc *)&ext->tx_desc[pidx];
		
		tfd->tdes1 = TDES1_CI | TDES1_ED | TDES1_BD | TDES1_CE | framelen;
		tfd->tdes0 = TDES0_OWN;
#if DEBUG_TX
dump_tx_desc(ext);
#endif
		wakeup = 1;
	}

	if (wakeup)
		OUT32(ext, CSR1, 0x1);		/* kick Tx poll demand */
										
	return( 0 ); 
}


/* entry from redirector */
int dm9pci_send_packets( npkt_t *npkt, void *hdl)
{
	Nic_t			*nic = (Nic_t *)hdl;
	Nic_Dm9pci_Ext_t	*ext;
	net_buf_t		*buf;
	io_net_msg_join_mcast_t *msg;

#if DEBUG_TX
fprintf(dfd, "dm9pci_send_packets()\n\r");
#endif
	ext = (Nic_Dm9pci_Ext_t *)nic->ext;

	if( npkt->flags & _NPKT_MSG )
	{
		buf = TAILQ_FIRST( &npkt->buffers );
		if( buf != NULL )
		{
			/* If selective mutlicasting worked on this card 
			   this is where it would happen */
			msg = (io_net_msg_join_mcast_t *)buf->net_iov->iov_base;
			if( msg )
			{
				pthread_mutex_lock( &ext->mutex );
				switch( msg->type )
				{
					case _IO_NET_JOIN_MCAST:
						break;

					case _IO_NET_REMOVE_MCAST:
						break;

					default:
						if( ext->verbose )
						{
							fprintf( stderr, "devn-dm9pci: Message type not supported.\n\r" );
						}
						break;
				}
				pthread_mutex_unlock( &ext->mutex );
			}
		}

		if( npkt->flags & _NPKT_NO_RES )
		{
			ion_free(npkt->org_data);
			ion_free( npkt );
		}
		else
		{
			ion_tx_complete(ext->reg_hdl, npkt);
		}
	}
	else
	{
		pthread_mutex_lock( &ext->mutex );

		if( ext->ntail ) {
			ext->ntail->next = npkt;
		}
		else {
			ext->nhead = npkt;
		}
//dump_npkt(npkt);
		while( npkt->next ) {
			npkt = npkt->next;
//dump_npkt(npkt);
		}
		ext->ntail = npkt;

		dm9pci_send( nic );
		pthread_mutex_unlock( &ext->mutex );
	}

	return( EOK );
}


int dm9pci_transmit_complete( Nic_t *nic )
{
	int		cidx;
	uint32_t	status;
	GenStats_t		*gstats;
	npkt_t		*npkt;
	npkt_t		*free_list_head;
	npkt_t		*free_list_tail;
	Nic_Dm9pci_Ext_t	*ext;

#if DEBUG_TX
fprintf(dfd, "dm9pci_transmit_complete()\n\r");
#endif
	ext = (Nic_Dm9pci_Ext_t *)nic->ext;
	gstats	= &nic->nstats.gstats;
	free_list_head = NULL;
	free_list_tail = NULL;

	do
	{
		cidx = (ext->tx_cidx + 1) % TX_DESC_NUM;
		status	= ext->tx_desc[cidx].tdes0;
		npkt = ext->tx_pktq[cidx];

		if ( !(status & TDES0_OWN) && npkt )
		{
			if (npkt->flags & _NPKT_NO_RES)
			{
				ion_free(npkt->org_data);
				ion_free(npkt);
			}
			else
			{
				if (free_list_tail)
					free_list_tail->next = npkt;
				else
					free_list_head = npkt;

				npkt->next = NULL;
				free_list_tail = npkt;
			}

			ext->tx_pktq[cidx] = NULL;
			if (!(status & TDES0_ES) )
				gstats->xmit_ok++;
			else
				gstats->xmit_error++;
			ext->tx_cidx = cidx;
		}
		else
			break;
	} while (ext->last_tx != ext->cur_tx );

	/* Free up these packets */
	if (free_list_head)
	{
		pthread_mutex_unlock( &ext->mutex );
		while (free_list_head)
		{
			npkt = free_list_head;
			free_list_head = free_list_head->next;
			ion_tx_complete(ext->reg_hdl, npkt);
		}
		pthread_mutex_lock( &ext->mutex );
	}
	
	dm9pci_send( nic );

	return( 0 );
}

#if CONSOLE_DEBUG
void dump_npkt(npkt_t *npkt)
{
	net_iov_t	*iov;
	net_buf_t	*buf;
	int i, j, l;
	char *ptr;
	int total = 0;

	for(l=0, buf = TAILQ_FIRST(&npkt->buffers); buf; buf = TAILQ_NEXT(buf, ptrs))
	{
		for(i=0, iov=buf->net_iov; i<buf->niov; i++, iov++)
		{
			for(j=0, ptr=(char *)iov->iov_base; j<iov->iov_len; j++)
			{
				if(total++ >= npkt->framelen)
					return;
				fprintf(dfd, "%02X ", (uint8_t)*ptr++);
				if(++l >= 16)
				{
					l = 0;
					fprintf(dfd, "\n\r");
				}
			}
		}
	}
fprintf(dfd, "\n\r");
}
#endif

int copy_npkt_data(void *addr, npkt_t *npkt)
{
	net_iov_t	*iov;
	net_buf_t	*buf;
	int i, datalen, len;
	char *ptr;

	ptr = (char *)addr;
	for(datalen=0, buf = TAILQ_FIRST(&npkt->buffers); buf; buf = TAILQ_NEXT(buf, ptrs))
	{
		for(i=0, iov=buf->net_iov; i<buf->niov; i++, iov++)
		{
			if ( (datalen + iov->iov_len) > ETH_MAX_PKT_LEN )
				len = ETH_MAX_PKT_LEN - datalen;
			else
				len = iov->iov_len;
			memcpy(ptr, (char *)iov->iov_base, len);
			ptr += len;
			datalen += len;
		}
	}

	if (datalen < ETH_MIN_PKT_LEN)
		datalen = ETH_MIN_PKT_LEN;

	return (datalen);
}